@@ -3,7 +3,7 @@
 // Author:      Robert Roebling
 // Modified by:
 // Created:     19.10.99
-// RCS-ID:      $Id: dndcmn.cpp,v 1.5 2004/05/23 20:52:00 JS Exp $
+// RCS-ID:      $Id: dndcmn.cpp 43664 2006-11-26 21:50:51Z JS $
 // Copyright:   (c) wxWidgets Team
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -12,10 +12,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "dndbase.h"
-#endif
-
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
@@ -24,4 +20,12 @@
 
 #include "wx/dnd.h"
 
-// Not much to do here, really, but GNU need this file.
+#if wxUSE_DRAG_AND_DROP
+
+bool wxIsDragResultOk(wxDragResult res)
+{
+    return res == wxDragCopy || res == wxDragMove || res == wxDragLink;
+}
+
+#endif
+
