@@ -2,24 +2,20 @@
 // Name:        gtk/timer.cpp
 // Purpose:     wxTimer implementation
 // Author:      Robert Roebling
-// Id:          $Id: timer.cpp,v 1.22 2005/03/21 23:42:19 VZ Exp $
+// Id:          $Id: timer.cpp 62333 2009-10-08 13:56:30Z VZ $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "timer.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
 #if wxUSE_TIMER
 
 #include "wx/timer.h"
+#include "wx/app.h"
 
-#include "gtk/gtk.h"
+#include <gtk/gtk.h>
 
 // ----------------------------------------------------------------------------
 // wxTimer
@@ -33,13 +29,14 @@ static gint timeout_callback( gpointer data )
     wxTimer *timer = (wxTimer*)data;
 
     // Don't change the order of anything in this callback!
-    
-    if (timer->IsOneShot())
+
+    const bool oneshot = timer->IsOneShot();
+    if ( oneshot )
     {
         // This sets m_tag to -1
         timer->Stop();
     }
-    
+
     // When getting called from GDK's timer handler we
     // are no longer within GDK's grab on the GUI
     // thread so we must lock it here ourselves.
@@ -50,7 +47,11 @@ static gint timeout_callback( gpointer data )
     // Release lock again.
     gdk_threads_leave();
 
-    if (timer->IsOneShot())
+    wxApp* app = wxTheApp;
+    if (app)
+        app->WakeUpIdle();
+
+    if ( oneshot )
         return FALSE;
 
     return TRUE;
@@ -73,9 +74,9 @@ bool wxTimer::Start( int millisecs, bool oneShot )
     (void)wxTimerBase::Start(millisecs, oneShot);
 
     if (m_tag != -1)
-        gtk_timeout_remove( m_tag );
+        g_source_remove( m_tag );
 
-    m_tag = gtk_timeout_add( m_milli, timeout_callback, this );
+    m_tag = g_timeout_add( m_milli, timeout_callback, this );
 
     return TRUE;
 }
@@ -84,7 +85,7 @@ void wxTimer::Stop()
 {
     if (m_tag != -1)
     {
-        gtk_timeout_remove( m_tag );
+        g_source_remove( m_tag );
         m_tag = -1;
     }
 }
